# -*- coding: utf-8 -*-

"""
Module description
"""
# TODO:
#   (+) 
#---------
# NOTES:
#   -
from smos import SMOSTs
import pandas as pd
import matplotlib.pyplot as plt

path_old = r'R:\Datapool\SMOS\02_processed\L3_SMOS_IC_Soil_Moisture\timeseries\ASC'
path_new = r"\\project10\data-write\USERS\wpreimes\smosic_ts\asc"

ds_new = SMOSTs(path_new, parameters=['Soil_Moisture'])
ds_old = SMOSTs(path_old, parameters=['Soil_Moisture'])

ts_old = ds_old.read(-14,14).rename(columns={'Soil_Moisture': 'old_sm'})
ts_new = ds_new.read(-14,14).rename(columns={'Soil_Moisture': 'new_sm'})

df = pd.concat([ts_old, ts_new+0.1], axis=1)
df.plot(marker='.')
plt.show()


# -*- coding: utf-8 -*-
# The MIT License (MIT)
#
# Copyright (c) 2019, TU Wien
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Module for a command line interface to convert the SMOS image data into a
time series format using the repurpose package
"""

import os
import sys
from datetime import datetime

from repurpose.img2ts import Img2Ts
from smos.smos_ic.interface_ic import SMOS_IC_Img, SMOS_IC_Ds
from smos.grid import EASE25CellGrid
from smos.reshuffle import firstfile, mkdate, str2bool, parse_args


def reshuffle(input_root, outputpath,
              startdate, enddate,
              imgbuffer=200, **ds_kwargs):
    """
    Reshuffle method applied to SMOS image data.

    Parameters
    ----------
    input_root: string
        input path where smos ic data was downloaded to (yearly folders)
    outputpath : string
        Output path.
    startdate : datetime
        Start date.
    enddate : datetime
        End date.
    imgbuffer: int, optional
        How many images to read at once before writing time series.
    ds_kwargs: dict
        Kwargs that are passed to the image datastack class
    """

    ff, file_vars = firstfile(input_root)
    fp, ff = os.path.split(ff)

    if 'grid' not in ds_kwargs.keys():
        ds_kwargs['grid'] = EASE25CellGrid(None)
    if 'parameters' not in ds_kwargs.keys():
        ds_kwargs['parameters'] = None

    # this is only for reading the ts_attrs
    input_dataset = SMOS_IC_Img(filename=os.path.join(fp, ff),
                                parameters=ds_kwargs['parameters'], flatten=True, read_flags=None,
                                grid=ds_kwargs['grid'])
    _, ts_attributes = input_dataset._read_img()
    global_attr = input_dataset.get_global_attrs()

    if ds_kwargs['parameters'] is None:
        ds_kwargs['parameters'] = input_dataset.parameters

    input_dataset = SMOS_IC_Ds(input_root, flatten=True, **ds_kwargs)

    if not os.path.exists(outputpath):
        os.makedirs(outputpath)

    # get time series attributes from first day of data.
    reshuffler = Img2Ts(input_dataset=input_dataset, outputpath=outputpath,
                        startdate=startdate, enddate=enddate,
                        input_grid=ds_kwargs['grid'].cut(),  # drop points that are not subset
                        imgbuffer=imgbuffer, cellsize_lat=5.0,
                        cellsize_lon=5.0, global_attr=global_attr, zlib=True,
                        unlim_chunksize=1000, ts_attributes=ts_attributes)
    reshuffler.calc()


def main(args):
    """
    Main routine used for command line interface.
    Parameters
    ----------
    args : list of str
        Command line arguments.
    """
    args = parse_args(args)

    input_grid = EASE25CellGrid(bbox=tuple(args.bbox) if args.bbox is not None else None)

    flags = (0) if args.only_good else (0, 1)

    ds_kwargs = {'read_flags': flags, 'grid': input_grid,
                 'parameters': args.parameters}

    reshuffle(args.dataset_root,
              args.timeseries_root,
              args.start,
              args.end,
              imgbuffer=args.imgbuffer,
              **ds_kwargs)


def run():
    main(sys.argv[1:])



"""
Module description
"""
# TODO:
#   (+) 
#---------
# NOTES:
#   -


from smecv_grid import SMECV_Grid_v052
from smos import SMOSImg
import matplotlib .pyplot as plt


# read one of the original files, also read the non-good values
smos_img = r"R:\Datapool_raw\SMOS\datasets\L3_SMOS_IC_Soil_Moisture\ASC\2014\SM_RE06_MIR_CDF3SA_20140107T000000_20140107T235959_105_001_8.DBL.nc"
reader = SMOSImg(filename=smos_img, read_flags=(0,1), parameters=['Soil_Moisture'])
image = reader.read()

fig1, ax1 = plt.subplots()
ax1.imshow(image.data['Soil_Moisture'])



# read one of the original files, only read the good values
smos_img = r"R:\Datapool_raw\SMOS\datasets\L3_SMOS_IC_Soil_Moisture\ASC\2014\SM_RE06_MIR_CDF3SA_20140107T000000_20140107T235959_105_001_8.DBL.nc"
reader = SMOSImg(filename=smos_img, read_flags=(0), parameters=['Soil_Moisture'])
image = reader.read()

fig2, ax2 = plt.subplots()
ax2.imshow(image['Soil_Moisture'])


# read one of the resampled files
grid = SMECV_Grid_v052(None)
smos_img = r"R:\Projects\CCIplus_Soil_Moisture\07_data\processed\smos_ic_resampled_quarter\ASC\2016\SMOS_IC_CCI_GRID_20160103T000000.nc"
reader = SMOSImg(filename=smos_img, read_flags=None, grid=grid, parameters=['smos_sm_resampled'])
image = reader.read()
fig3, ax3 = plt.subplots()
ax3.imshow(image['smos_sm_resampled'])


##############################################################################
# reshuffle the resampled files
from smos import reshuffle
from datetime import datetime
import tempfile
from pygeogrids.grids import BasicGrid

input_root = r"R:\Projects\CCIplus_Soil_Moisture\07_data\processed\smos_ic_resampled_quarter\ASC"
outputpath = tempfile.mkdtemp()
startdate, enddate = datetime(2017,12,1), datetime(2018,1,31)
gpis, lons, lats, cells = SMECV_Grid_v052(None).get_grid_points()
cci_grid = BasicGrid(lons.filled(), lats.filled(), gpis.filled()).to_cell_grid(5.)

ds_kwargs = {'grid':cci_grid, 'read_flags': None,
            'filename_templ':'SMOS_IC_CCI_GRID_{datetime}T000000.nc'}
reshuffle(input_root, outputpath,
              startdate, enddate, parameters=['smos_sm_resampled'],
              ds_kwargs=ds_kwargs,
              imgbuffer=50)

# read the time series
from smos import SMOSTs
ds = SMOSTs(outputpath)
time_series = ds.read(-72,-4.821).dropna()
print(time_series)


# -*- coding: utf-8 -*-

# The MIT License (MIT)
#
# Copyright (c) 2021, TU Wien
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import numpy as np
from netCDF4 import Dataset
from smos.grid import EASE25CellGrid
from smos.interface import SMOSImg, SMOSDs


class SMOS_L4_Img(SMOSImg):
    """
    Child-class for reading one SMOS L4 CATDS - CESBIO RZSM netcdf image file.

    Parameters
    ----------
    filename: str
        filename of the SMOS nc image file
    mode: str, optional (default: 'r')
        mode of opening the file, only 'r' is implemented at the moment
    parameters : str or list, optional (default: None)
        one or list of parameters to read. We add 'Quality_Flags' if 'read_flags'
        is not None. All parameters are described in docs/varnames.rst.
        If None is passed, all parameters are read.
    flatten: bool, optional (default: False)
        If set then the data is read into 1D arrays. This is used to e.g
        reshuffle the data.
    grid : pygeogrids.CellGrid, optional (default: EASE25CellGrid)
        Grid that the image data is organised on, by default the global EASE25
        grid is used.
    read_flags : tuple, list, np.array or None, optional (default: np.linspace(0,1,6,endpoint=True))
        Filter values to read based on the selected quality flags.
        Values for locations that are not assigned any of the here passed flags
        are replaced with NaN (by default only the missing-data, i.e. flag=-1,
        are filtered out). If None is passed, no flags are considered.
    oper : bool, optional (default: False)
        Boolean operator distinguishing between the SMOS L4 RZSM Scientific and Operational
        products. Distinction is made due to differences in quality flag variable naming, values
        and their significance (see docs/varnames.rst).
    float_fillval : float or None, optional (default: np.nan)
        Fill Value for masked pixels, this is only applied to float variables.
        Therefore e.g. mask variables are never filled but use the fill value
        as in the data.
    """

    def __init__(self, filename, mode='r', parameters=None, flatten=False,
                 grid=EASE25CellGrid(bbox=None), read_flags=np.linspace(0,1,6,endpoint=True),
                 oper=False, float_fillval=np.nan):

        super().__init__(filename, mode=mode)

        if parameters is None:
            parameters = []
        if type(parameters) != list:
            parameters = [parameters]

        self.read_flags = read_flags
        self.parameters = parameters
        self.flatten = flatten

        self.oper = oper

        self.grid = grid

        self.image_missing = False
        self.img = None  # to be loaded
        self.glob_attrs = None

        self.float_fillval = float_fillval


    def _read_img(self) -> (dict, dict):
        # Read a netcdf image and metadata

        ds = Dataset(self.filename)
        self.glob_attrs = ds.__dict__

        param_img = {}
        param_meta = {}

        if len(self.parameters) == 0:
            # all data vars, exclude coord vars
            self.parameters = [k for k in ds.variables.keys() if
                               ds.variables[k].ndim != 1]

        parameters = list(self.parameters)

        if not self.oper:
            if (self.read_flags is not None) and ('QUAL' not in parameters):
                parameters.append('QUAL')
        else:
            if (self.read_flags is not None) and ('Quality' not in parameters):
                parameters.append('Quality')

        for parameter in parameters:
            metadata = {}
            param = ds.variables[parameter]
            data = param[:]

            # read long name, FillValue and unit
            for attr in param.ncattrs():
                metadata[attr] = param.getncattr(attr)

            data.mask = ~np.isfinite(data)
            np.ma.set_fill_value(data, metadata['_FillValue'])

            metadata['image_missing'] = 0

            param_img[parameter] = data
            param_meta[parameter] = metadata

        # filter with the flags (this excludes non-land points as well)
        if self.read_flags is not None:
            if not self.oper:
                flag_mask = ~np.isin(param_img['QUAL'], self.read_flags)
            else:
                flag_mask = ~np.isin(param_img['Quality'], self.read_flags)

        else:
            flag_mask = np.full(param_img[parameters[0]].shape, False)

        for param, data in param_img.items():

            param_img[param].mask = (data.mask | flag_mask)

            if self.float_fillval is not None:
                if issubclass(data.dtype.type, np.floating):
                    param_img[param] = data.filled(fill_value=self.float_fillval)

            param_img[param] = param_img[param].flatten()[self.grid.activegpis]

        if not self.oper:
            if ('QUAL' in param_img.keys()) and \
                    ('QUAL' not in self.parameters):
                param_img.pop('QUAL')
                param_meta.pop('QUAL')
        else:
            if ('Quality' in param_img.keys()) and \
                    ('Quality' not in self.parameters):
                param_img.pop('Quality')
                param_meta.pop('Quality')

        return param_img, param_meta


class SMOS_L4_Ds(SMOSDs):
    """
    Class for reading SMOS L4 CATDS - CESBIO RZSM images in nc format. Images are orgnaised in subdirs
    for each year.

    Parameters
    ----------
    data_path : str
        Path to the nc files
    parameters : str or list, optional (default: None)
        one or list of parameters to read. We add quality flags if 'read_flags'
        is not None. All parameters are described in docs/varnames.rst.
        If None is passed, all parameters are read.
    flatten: bool, optional (default: False)
        If set then the data is read into 1D arrays. This is used to e.g
        reshuffle the data.
    grid : pygeogrids.CellGrid, optional (default: EASE25CellGrid)
        Grid that the image data is organised on, by default the global EASE25
        grid is used.
    read_flags : tuple, list, np.array or None, optional (default: np.linspace(0,1,6,endpoint=True))
        Filter values to read based on the selected quality flags.
        Values for locations that are not assigned any of the here passed flags
        are replaced with NaN (by default only the missing-data, i.e. flag=-1,
        are filtered out). If None is passed, no flags are considered.
    oper : bool, optional (default: False)
        Boolean operator distinguishing between the SMOS L4 RZSM Scientific and Operational
        products. Distinction is made due to differences in quality flag variable naming, values
        and their significance (see docs/varnames.rst).
    float_fillval : float or None, optional (default: np.nan)
        Fill Value for masked pixels, this is only applied to float variables.
        Therefore e.g. mask variables are never filled but use the fill value
        as in the data.
    """

    default_fname_template = "SM_*_MIR_CLF4RD*_{datetime}T000000_{datetime}T235959_*_*_*.DBL.nc"

    def __init__(self, data_path, parameters=None, flatten=False,
                 grid=EASE25CellGrid(bbox=None), filename_templ=None,
                 read_flags=np.linspace(0,1,6,endpoint=True), oper=False, float_fillval=np.nan):

        if filename_templ is None:
            filename_templ = self.default_fname_template

        super().__init__(data_path, ioclass=SMOS_L4_Img,
                         parameters=parameters,
                         flatten=flatten,
                         grid=grid,
                         filename_templ=filename_templ,
                         read_flags=read_flags,
                         float_fillval=float_fillval,
                         additional_kws={'oper': oper}
                         )



import os
import pandas as pd
from tempfile import TemporaryDirectory
from smos.smos_l2.reshuffle import swath2ts, extend_ts
from smos.misc import read_summary_yml
from pynetcf.time_series import GriddedNcIndexedRaggedTs
from pygeogrids.netcdf import load_grid
import numpy as np

def test_reshuffle_and_update():
    img_path = os.path.join(os.path.join(os.path.dirname(__file__), '..', 'smos-test-data', 'L2_SMOS'))
    with TemporaryDirectory() as ts_path:
        swath2ts(img_path, ts_path, startdate='2022-01-01',  enddate='2022-01-02')  # enddate is excluded

        assert os.path.isfile(os.path.join(ts_path, 'grid.nc'))
        props = read_summary_yml(ts_path)
        assert pd.to_datetime(props['last_day']) == pd.to_datetime('2022-01-02')

        grid = load_grid(os.path.join(ts_path, 'grid.nc'))
        reader = GriddedNcIndexedRaggedTs(ts_path, grid=grid)

        ts = reader.read(74.958, 14.923)
        assert len(ts.index) == 1
        np.testing.assert_almost_equal(
            ts.loc['2022-01-01', 'Soil_Moisture'].values[0],
            0.236319, 5
        )

        ts = reader.read(-70.696, 50.629)
        assert len(ts) == 1

        reader.close()  # not great for production...

        extend_ts(img_path, ts_path)
        props = read_summary_yml(ts_path)
        assert pd.to_datetime(props['last_day']) == pd.to_datetime('2022-01-03')

        reader = GriddedNcIndexedRaggedTs(ts_path, grid=grid)

        ts = reader.read(-70.696, 50.629)
        np.testing.assert_almost_equal(
            ts.loc['2022-01-02', 'Soil_Moisture'].values[0],
            0.52442, 5
        )
        assert 1 in ts.index.day
        assert 2 in ts.index.day
        assert 3 not in ts.index.day  # this must be excluded

        assert len(ts) == 2

        reader.close()



import pandas as pd
import os
from datetime import date
import typing as t
import yaml

def _get_first_and_last_file(path: str):
    # Get list of all years (folders) in the path
    years = sorted([folder for folder in os.listdir(path) if folder.isdigit()], key=int)

    if not years:
        return None, None

    # Get the first year and last year
    first_year = years[0]
    last_year = years[-1]

    # Handle the first year
    first_year_path = os.path.join(path, first_year)
    first_months = sorted([folder for folder in os.listdir(first_year_path) if folder.isdigit()], key=int)

    if first_months:
        first_month = first_months[0]
        first_month_path = os.path.join(first_year_path, first_month)
        first_days = sorted([folder for folder in os.listdir(first_month_path) if folder.isdigit()], key=int)

        if first_days:
            first_day = first_days[0]
            first_day_path = os.path.join(first_month_path, first_day)
            first_files = sorted(os.listdir(first_day_path))
            first_file = first_files[0] if first_files else None
        else:
            first_day_path = first_month_path
            first_files = sorted(os.listdir(first_day_path))
            first_file = first_files[0] if first_files else None
    else:
        first_month_path = first_year_path
        first_files = sorted(os.listdir(first_month_path))
        first_file = first_files[0] if first_files else None

    # Handle the last year
    last_year_path = os.path.join(path, last_year)
    last_months = sorted([folder for folder in os.listdir(last_year_path) if folder.isdigit()], key=int, reverse=True)

    if last_months:
        last_month = last_months[0]
        last_month_path = os.path.join(last_year_path, last_month)
        last_days = sorted([folder for folder in os.listdir(last_month_path) if folder.isdigit()], key=int, reverse=True)

        if last_days:
            last_day = last_days[0]
            last_day_path = os.path.join(last_month_path, last_day)
            last_files = sorted(os.listdir(last_day_path))
        else:
            last_day_path = last_month_path
            last_files = sorted(os.listdir(last_day_path))
    else:
        last_month_path = last_year_path
        last_files = sorted(os.listdir(last_month_path))

    return first_file, last_files[-1] if last_files else None


def _get_date(f: str) -> t.Union[date, None]:
    for e in f.split('_'):
        try:
            dt = pd.to_datetime(e).to_pydatetime().date()
            return dt
        except Exception:
            continue
    return None


def get_first_last_day_images(img_path: str) -> \
        (t.Union[date, None], t.Union[date, None]):
    f, l = _get_first_and_last_file(img_path)
    first_day = _get_date(f) if f is not None else f
    last_day = _get_date(l) if l is not None else f

    return first_day, last_day

def read_summary_yml(path: str) -> dict:
    """
    Read image summary and return fields as dict.
    """
    path = os.path.join(path, 'overview.yml')

    with open(path, 'r') as stream:
        props = yaml.safe_load(stream)

    return props


"""
SMOS L3 Ascending and Descending, daily products, opearive and reprocessed, (A:ascending=6am and D:descending=6pm)

Downloaded from CATDS:

ftp.ifremer.fr
user : ext-catds-cpdc
p : catds2010

on 09-05-2022

Note:

* OPER(ative) and REP(rocessed) are mixed in the MIR_CLF3SA (Ascending) and MIR_CLF3SD (Descending) folders
"""


# -*- coding: utf-8 -*-

"""
Module description
"""
# TODO:
#   (+) 
#---------
# NOTES:
#   -

from smos.smos_ic import SMOSDs
from datetime import datetime
from smos.grid import EASE25CellGrid

path = r"R:\Datapool\SMOS\01_raw\datasets\L3_SMOS_IC_Soil_Moisture\DES"

grid_senegal = EASE25CellGrid(bbox= (-17.625, 12.125, -11.125, 16.875))
grid_austria = EASE25CellGrid(bbox= (9.375, 46.375, 17.125, 49.125))
grid_mozambique = EASE25CellGrid(bbox= (30.125, -26.875, 40.875, -10.375))
grid_morocco = EASE25CellGrid(bbox= (-17.125,21.375, -1.125, 35.875))
grid_eu = EASE25CellGrid(bbox=(-11., 34., 43., 71.))

grids = {'Sengal': grid_senegal, 'Austria': grid_austria,
         'Mozambique': grid_mozambique, 'Morocco': grid_morocco}

for country, grid in grids.items():

    ds = SMOSDs(path, grid=grid, read_flags=(0,1))

    ds.write_multiple(r'R:\Projects\SMART-DRI\08_scratch\smos_ic_stacks\DES',
                      start_date=datetime(2010,1,1), end_date=datetime(2018,12,31),
                      stackfile=f'SMOS_IC_DES_{country}_stack.nc')


# -*- coding: utf-8 -*-

# The MIT License (MIT)
#
# Copyright (c) 2021, TU Wien
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import numpy as np
from netCDF4 import Dataset, date2num, num2date
from smos.grid import EASE25CellGrid
from smos.interface import SMOSImg, SMOSDs


class SMOS_IC_Img(SMOSImg):
    """
    Class for reading one SMOS IC netcdf image file.

    Parameters
    ----------
    filename: str
        filename of the SMOS nc image file
    mode: str, optional (default: 'r')
        mode of opening the file, only 'r' is implemented at the moment
    parameters : str or list, optional (default: None)
        one or list of parameters to read. We add 'Quality_Flags' if 'read_flags'
        is not None. All parameters are described in docs/varnames.rst. If
        None are passed, all parameters are read.
    flatten: bool, optional (default: False)
        If set then the data is read into 1D arrays. This is used to e.g
        reshuffle the data.
    grid : pygeogrids.CellGrid, optional (default: EASE25CellGrid)
        Grid that the image data is organised on, by default the global EASE25
        grid is used.
    read_flags : tuple or None, optional (default: (0, 1))
        Filter values to read based on the selected QUALITY_FLAGS.
        Values for locations that are not assigned any of the here passed flags
        are replaces with NaN (by default only the missing-data, i.e. flag=2,
        locations are filtered out). If None is passed, no flags are considered.
    float_fillval : float or None, optional (default: np.nan)
        Fill Value for masked pixels, this is only applied to float variables.
        Therefore e.g. mask variables are never filled but use the fill value
        as in the data.
    """

    def __init__(self, filename, mode='r', parameters=None, flatten=False,
                 grid=EASE25CellGrid(bbox=None), read_flags=(0, 1), float_fillval=np.nan):

        super(SMOSImg, self).__init__(filename, mode=mode)

        if parameters is None:
            parameters = []
        if type(parameters) != list:
            parameters = [parameters]

        self.read_flags = read_flags
        self.parameters = parameters
        self.flatten = flatten

        self.grid = grid

        self.image_missing = False
        self.img = None  # to be loaded
        self.glob_attrs = None

        self.float_fillval = float_fillval


    def _read_img(self) -> (dict, dict):
        # Read a netcdf image and metadata

        ds = Dataset(self.filename)
        self.glob_attrs = ds.__dict__

        param_img = {}
        param_meta = {}

        if len(self.parameters) == 0:
            # all data vars, exclude coord vars
            self.parameters = [k for k in ds.variables.keys() if
                               ds.variables[k].ndim != 1]

        parameters = list(self.parameters)

        if (self.read_flags is not None) and ('Quality_Flag' not in parameters):
            parameters.append('Quality_Flag')

        for parameter in parameters:
            metadata = {}
            param = ds.variables[parameter]
            data = param[:]

            # read long name, FillValue and unit
            for attr in param.ncattrs():
                metadata[attr] = param.getncattr(attr)

            data.mask = ~np.isfinite(data)
            np.ma.set_fill_value(data, metadata['_FillValue'])

            metadata['image_missing'] = 0

            param_img[parameter] = data
            param_meta[parameter] = metadata

        # filter with the flags (this excludes non-land points as well)
        if self.read_flags is not None:
            flag_mask = ~np.isin(param_img['Quality_Flag'], self.read_flags)
        else:
            flag_mask = np.full(param_img[parameters[0]].shape, False)

        for param, data in param_img.items():

            param_img[param].mask = (data.mask | flag_mask)

            if self.float_fillval is not None:
                if issubclass(data.dtype.type, np.floating):
                    param_img[param] = data.filled(fill_value=self.float_fillval)

            param_img[param] = param_img[param].flatten()[self.grid.activegpis]

        if ('Quality_Flag' in param_img.keys()) and \
                ('Quality_Flag' not in self.parameters):
            param_img.pop('Quality_Flag')
            param_meta.pop('Quality_Flag')

        return param_img, param_meta


class SMOS_IC_Ds(SMOSDs):
    """
    Class for reading SMOS IC images in nc format. Images are orgnaised in subdirs
    for each year.

    Parameters
    ----------
    data_path : str
        Path to the nc files
    parameter : str or list, optional (default: None)
        one or list of parameters to read, see SMOS documentation
        for more information (default: 'Soil_Moisture').
    flatten: bool, optional (default: False)
        If set then the data is read into 1D arrays. This is used to e.g
        reshuffle the data.
    grid : pygeogrids.CellGrid, optional (default: EASE25CellGrid)
        Grid that the image data is organised on, by default the global EASE25
        grid is used.
    read_flags : tuple or None, optional (default: (0, 1))
        Filter values to read based on the selected QUALITY_FLAGS.
        Values for locations that are not assigned any of the here passed flags
        are replaces with NaN (by default only the missing-data, i.e. flag=2,
        locations are filtered out).
    float_fillval : float or None, optional (default: np.nan)
        Fill Value for masked pixels, this is only applied to float variables.
        Therefore e.g. mask variables are never filled but use the fill value
        as in the data.
    """

    default_fname_template = "SM_RE06_MIR_CDF3S*_{datetime}T000000_{datetime}T235959_105_*_8.DBL.nc"

    def __init__(self, data_path, parameters=None, flatten=False,
                 grid=EASE25CellGrid(bbox=None), filename_templ=None,
                 read_flags=(0, 1), float_fillval=np.nan):

        if filename_templ is None:
            filename_templ = self.default_fname_template

        super().__init__(data_path, ioclass=SMOS_IC_Img,
                         parameters=parameters,
                         flatten=flatten,
                         grid=grid,
                         filename_templ=filename_templ,
                         read_flags=read_flags,
                         float_fillval=float_fillval
                         )
